﻿
init python:

    class dance_gain(renpy.Displayable):
        def __init__(self):
            super(renpy.Displayable,self).__init__()
            self.text = ""
            self.x = 0
            self.y = 0
            self.duration = 20
            self.direction = 3
            self.visible = True
        def show(self, text, x, y, direction = -4):
            self.text = str(text)
            self.x = x
            self.y = y
            self.direction = direction
            self.duration = 20
            self.visible = True

        def render(self,width,height,st,at):
            if self.visible:
                if self.duration < 0:
                    self.visible = False
                self.y += self.direction
                self.duration -= 1
                img = Text(self.text, size = 30)
            else:
                img = Text("", size = 30)
            t = Transform(img)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

    class dance_object(renpy.Displayable):
        def __init__(self, image, x, y, parent):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.x = x
            self.y = y
            self.count = 0
            self.visible = True
        
        def render(self,width,height,st,at):
            if self.visible:
                if self.count > 50:
                    self.visible = False
                self.count += 1
                t = Transform(self.image)
                child_render = renpy.render(t, width, height, st, at)
                cw, ch = child_render.get_size()

                rv = renpy.Render(cw, ch)
                rv.blit(child_render, (self.x, self.y))

                renpy.redraw(self, 0)
                return rv
            return renpy.Render(0,0)
    class dance_arrow(renpy.Displayable):
        def __init__(self, image, direction, parent):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.direction = direction
            self.parent = parent

            self.color = "beige"
            self.moving = True
            self.x = 0
            self.y = 0
            self.hit = False
        
        def render(self,width,height,st,at):
            if self.moving:
                self.y += 4
                if self.y > 850:
                    self.color = "red"
                    if self.y > 950:
                        self.moving = False
                        self.parent.miss(self)
            img = "{} {} {}".format(self.image, self.direction, self.color)
            t = Transform(img)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

    class dance_class:
        def __init__(self, music, recipe):
            self.music = music
            self.recipe = recipe
            self.moves = []
            self.misses = 0
            self.score = 0
            self.gain = dance_gain()
            self.current_move = "idle"
            self.wait = 0
            self.move_wait = 0
            self.objects = []
            self.buttons = [
                [["w", "K_UP"], 200, 800],
                [["s", "K_DOWN"], 300, 800],
                [["d", "K_RIGHT"], 400, 800],
                [["a", "K_LEFT"], 100, 800],
            ]
        def remove_object(self, object):
            self.objects.remove()
        def start(self):
            pass
        def clean(self):
            moves = [i for i in self.moves if i.moving]
            objects = [i for i in self.objects if i.visible]
            self.objects = objects
            self.moves = moves
        def move(self, move):
            self.current_move = move[0][0]
            direction = -4
            for i in self.moves:
                if i.moving:
                    if i.direction == move[0][0]:
                        dist = distance(i.y, 800)
                        if dist < 5:
                            i.color = "green"
                            i.moving = False
                            color = "gold"
                            score = (50-dist)*2
                            break
                        elif dist < 30:
                            i.color = "green"
                            i.moving = False
                            color = "green"
                            score = 50-dist
                            break
            else:
                color = "red"
                self.current_move = "miss"
                score = -200
                direction = 4
            self.score += score
            x = move[1]
            y = move[2]
            self.gain.show(score, x, y, direction)
            o = dance_object("dance_circle {}".format(color), x, y, self)
            self.objects.append(o)
            

        def tick(self):
            self.wait += 1
            
            if not self.current_move == "idle":
                self.move_wait += 1
                if self.move_wait > 3:
                    self.current_move = "idle"
                    self.move_wait = 0
            if self.wait > 4:
                chance = random.randint(0,100)
                if chance < 70:
                    self.spawn()
                    self.wait = 0

        def spawn(self):
            self.clean()
            direction = random.choice(self.buttons)
            move = dance_arrow("dance_arrow", direction[0][0], self)
            move.x = direction[1]
            self.moves.append(move)
        def miss(self, arrow):
            self.gain.show(-100, arrow.x, arrow.y, 4)
            self.misses += 1
            self.score += -100
            self.clean()

transform dance_bob(speed):
    ease speed yoffset 10
    ease speed yoffset 0
    repeat

screen dance_screen(base  = "dancer", speed = .2):
    default g = dance_class(None,None)

    add "{} {}".format(base, g.current_move) align .8,1.0 at dance_bob(speed)

    for i in g.moves:
        add i

    for n,i in enumerate(g.buttons):
        button:
            padding 0,0 background None xpos i[1] ypos i[2]
            add "dance_circle"
            action Function(g.move, i) keysym i[0]

    timer speed repeat True action Function(g.tick)
    for i in g.objects:
        add i
    # vbox:
    #     text str(len(g.moves))
    #     text str(len(g.objects))
    text str(g.score) size 50 outlines([(10, "#1e0736db", 0, 0)]) align .5,.05
    add g.gain

init:
    image dance_arrow w beige:
        Crop((0,0,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow s beige:
        Crop((32,0,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow d beige:
        Crop((64,0,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow a beige:
        Crop((96,0,32,32), "dance/sprites.png")
        zoom 3 nearest True

    image dance_arrow w green:
        Crop((0,32,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow s green:
        Crop((32,32,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow d green:
        Crop((64,32,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow a green:
        Crop((96,32,32,32), "dance/sprites.png")
        zoom 3 nearest True

    image dance_arrow w red:
        Crop((0,64,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow s red:
        Crop((32,64,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow d red:
        Crop((64,64,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_arrow a red:
        Crop((96,64,32,32), "dance/sprites.png")
        zoom 3 nearest True

    image dance_circle:
        Crop((0,96,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_circle green:
        Crop((32,96,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_circle red:
        Crop((64,96,32,32), "dance/sprites.png")
        zoom 3 nearest True
    image dance_circle gold:
        Crop((96,96,32,32), "dance/sprites.png")
        zoom 3 nearest True

    image bg dance_floor:
        rotate_pad False
        align (0.5, 0.5)
        "bg dance"
        parallel:
            ease 2 zoom 1.1
            ease 2 zoom 1.2
            repeat
        parallel:
            ease .4 rotate -1
            ease .4 rotate 1
            repeat

label dance_example:
    scene black
    show bg dance_floor with dissolve
    play music "dance/Dance With Us.mp3"
    call screen dance_screen

